import 'package:sa_flutter/service.base.dart';

class CrudRequestRoles {
  static const String USER = 'user';
  static const String ADMIN = 'admin';
  static const String SAMDIN = 'sadmin';
  static const String none = '';
}

abstract class CrudBase<T> extends ServiceBase<T> {
  final WithTokenGetter tokenGetter;

  CrudBase(
    this.tokenGetter,
    String baseUrl,
    String path,
    String requestRole,
  ) : super(
          baseUrl,
          path,
          requestRole: requestRole ?? CrudRequestRoles.USER,
        );

  getToken() {
    return tokenGetter.getToken();
  }

  String formatPathFromParts(List<String> parts) {
    parts.where((element) => element != null);
    return parts.join('/');
  }

  Future<T> getOne(String id, {String callRole}) {
    String formattedPath =
        formatPathFromParts([baseUrl, callRole ?? requestRole, path, id]);
    return this.get(formattedPath);
  }

  Future<T> getAll({Map params = const {}, String callRole}) {
    String formattedPath =
        formatPathFromParts([baseUrl, callRole ?? requestRole, path]);
    return this.get(formattedPath);
  }

  Future<T> create(T item, {String callRole}) {
    String formattedPath =
        formatPathFromParts([baseUrl, callRole ?? requestRole, path]);
    return this.post(formattedPath, item);
  }

  Future<T> edit(String id, T item, {String callRole}) {
    String formattedPath =
        formatPathFromParts([baseUrl, callRole ?? requestRole, path, id]);
    return this.put(formattedPath, item);
  }

  Future<T> delete(String id, {String callRole}) {
    String formattedPath =
        formatPathFromParts([baseUrl, callRole ?? requestRole, path, id]);
    return this.delete(formattedPath);
  }

  @override
  mapper(data) {
    return data;
  }
}
