import 'dart:convert';
import 'package:http/http.dart' as http;

class WithTokenGetter {
  Future<String> getToken() {}
}

abstract class ServiceBase<T> implements WithTokenGetter {
  String path = '';
  String requestRole = 'user';
  final String baseUrl;
  ServiceBase(this.baseUrl, this.path, {this.requestRole});

  Future<String> getToken();
  T mapper(dynamic data);

  Future<Map<String, String>> getHeaders({
    String contentType = 'application/json',
  }) async {
    String token = await this.getToken();
    Map<String, String> baseHeaders = {
      'content-type': contentType,
      'accept': 'application/ld+json',
    };

    if (token != null) {
      baseHeaders['authorization'] = 'Bearer $token';
    }

    return baseHeaders;
  }

  dynamic handleResponse(http.Response response) {
    if (response.statusCode < 300) {
      return response.body.isNotEmpty
          ? json.decode(response.body)
          : response.body;
    }

    String desc = json.decode(response.body)["message"];
    throw Exception(desc ?? response.body);
  }

  /// Methods
  Future<T> get(String path) async {
    var headers = await this.getHeaders();
    final response = await http.get(
      path,
      headers: headers,
    );

    var data = handleResponse(response);
    return mapper(data);
  }

  Future<T> post(String path, dynamic item) async {
    final response = await http.post(
      path,
      body: json.encode(item),
      headers: await this.getHeaders(),
    );
    var data = handleResponse(response);
    return this.mapper(data);
  }

  Future<T> put(String path, dynamic item) async {
    final response = await http.put(
      path,
      body: json.encode(item),
      headers: await this.getHeaders(),
    );

    var data = handleResponse(response);
    return this.mapper(data);
  }

  Future delete(String path) async {
    final response = await http.delete(
      path,
      headers: await this.getHeaders(),
    );

    return handleResponse(response);
  }
}
